.TH DbgLog 3 "2007-05-20" Unix Libraries


.SH NAME

.I DbgLog
-  Advanced Debugging Messages Filter and Logger


.SH DESCRIPTION

.R The
.B DbgLog
library filters and logs debugging/error messages according to
.I errno
, calling module/function, warning keyword and debugging level.
Particular errno/keyword can be made fatal. Output log files depend on
keyword.

Typical output looks like:

.RS
.PD 0
IPC.....<04>\ 1016120237s|706271us\ [000004]\ :\ 0003\ @\ main\ >\ \ ///
.P
-----------------------------------------------------------
.P
  dbg    dbg         time           msg     errno  module
.P
 keywd  level    seconds|milisec   number

.sp
///\ No\ such\ process\ (0000000543)
.P
    ----------------------------
.P
     errno message    opt args
.PD
.RE

.R But the output format can also be changed at will by setting a format
string.

.R This tool offers a tunable insight into the hierarchy of methods called
within the libraries using it. It provides a way to trace and debug invalid 
data as well as misbehaving functions. The ability to turn off or filter 
whole segments of the debugging traces ensures that debugging will not be 
hindered by useless data. It filters output much in the same way as system 
calls debugging tools (such as strace) allow. 


.SH PROGRAMMING INTERFACE

.R The API definition is entirely contained in the
.I <DbgLog.h>
include file.

.SS Overview

.R At the development stage, the main entry point in the API is the
.B DbgLog
funtion, that generates a log entry. To do that, this call must be supplied
with contextual information, like function name, type of entry, and severity
level. While this may seem cumbersome at first, this call needs only be used
in error contexts, meaning its use is not too frequent.

.R The synopsis of the variadic function
.B DbgLog
is:

.BI "#include " "<DbgLog.h>"

.I void
.B DbgLog
.R (
.I const char *
.B mod,
.I const char *
.B key,
.I unsigned char
.B lev,
.I const char *
.B mes, ...
.R ) ;

.I mod
is the calling module or function name. This is the main context
data, as this information will structure the actual hierarchy of calls.
.I key
is a debugging key, an identifier tag that will direct logging and
filtering of this message.
.I lev
is a numeric indication of severity (the lower, the more severe, with
0 having sense as "system-level error").

.R At the degugging stage, several calls provide the tools to fine-tune the logging output:
.BR DbgLog_SetFormat ", " DbgLog_SetLogFd ", " DbgLog_UpDbgLvl " and " DbgLog_DnDbgLvl ", " DbgLog_AddFatal " and " DbgLog_DelFatal.
.R It is also possible to set the active degugging levels with a simple
.I .properties
file, using the
.B DbgCfg
library
.RB "(see " DbgCfg(3) ")"

.B DbgLog_SetFormat
has a single string argument, determining the format
of log entries. The remaining functions have a debug key as first argument,
standing for a particular class of filtering and, more generally, handling
of that class of messages. The second argument is, for
.B DbgLog_SetLogFd
, an output file descriptor in which the messages will be written. The
.B ...DbgLvl
calls use a binary mask of levels (between 0 and 31) to set on or off. The
.B ...Fatal
calls set or unset some system error values to trigger immediate termination.




.sp

.sp


.SS Output format control
.sp

.TP 2
.B Function DbgLog_SetFormat


.I void
.B DbgLog_SetFormat
(
.I char
.B *
.B fmt
) ;

.PD 1
.P
.PD
Sets the format of output messages.

.B
	Args
.RS
.IP _ 2
.I fmt
	: format string
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
The format string is printed as-is for each log entry, except for escape sequences
of the form '%d*?', where d* desings an optionnal decimal number indicating the size
of this field, and ? is a character indicating the field type. Here is a list of
valid types, and their function:
.IP _ 2
.I %
	: a litteral '%' character
.IP "" 0
.IP _ 2
.I k
	: log key
.IP "" 0
.IP _ 2
.I l
	: log level
.IP "" 0
.IP _ 2
.I s
	: time of error (seconds component)
.IP "" 0
.IP _ 2
.I u
	: time of error (microseconds component)
.IP "" 0
.IP _ 2
.I n
	: entry number
.IP "" 0
.IP _ 2
.I c
	: calling function or module
.IP "" 0
.IP _ 2
.I e
	: error number (LibC errno)
.IP "" 0
.IP _ 2
.I r
	: error message related to the error number
.IP "" 0
.IP _ 2
.I m
	: user message
.IP "" 0
The default format is '%8k<%2l> %10ss|%6uus [%6n] : %4e @ %c > %r (%m)'.
.RE
.PD 1
.P
.PD





.SS Filters control
.sp

.TP 2
.B Function DbgLog_SetLogFd


.I void
.B DbgLog_SetLogFd
(
.I char
.B *
.B key
,
.I int
.B fd
) ;

.PD 1
.P
.PD
Sets log file descriptor.

.B
	Args
.RS
.IP _ 2
.I key
	: filter key identifier
.IP "" 0
.IP _ 2
.I fd
	: file descriptor representing an open stream to use to print traces
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If the key argument is the empty string, it designs the default (generic) filter,
whose caracteristics are used as a basis for new filters, or when filter is unspecified.
When the key is NULL, it designs ALL filters except the default.
.RE
.PD 1
.P
.PD




.TP 2
.B Function DbgLog_DbgLvl


.I unsigned
.I long
.B DbgLog_DbgLvl
(
.I char
.B *
.B key
) ;

.PD 1
.P
.PD
Gets the state of debugging levels.

.B
	Args
.RS
.IP _ 2
.I key
	: filter key identifier
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Returns
.RS
.IP _ 2
.I 0
	: if key was NULL, or did not refer to an existing filter
.IP "" 0
else the current mask of active debugging levels.
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If the key argument is the empty string, it designs the default (generic) filter,
whose caracteristics are used as a basis for new filters, or when filter is unspecified.
When the key is NULL, it designs ALL filters except the default.
The levels mask set bits positions (as in '<<') design debugging levels set ON.
.RE
.PD 1
.P
.PD




.TP 2
.B Function DbgLog_UpDbgLvl


.I void
.B DbgLog_UpDbgLvl
(
.I char
.B *
.B key
,
.I unsigned
.I long
.B lev
) ;

.PD 1
.P
.PD
Sets debugging levels ON.

.B
	Args
.RS
.IP _ 2
.I key
	: filter key identifier
.IP "" 0
.IP _ 2
.I lev
	: debugging levels mask
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If the key argument is the empty string, it designs the default (generic) filter,
whose caracteristics are used as a basis for new filters, or when filter is unspecified.
When the key is NULL, it designs ALL filters except the default.
The levels mask set bits positions (as in '<<') design debugging levels set ON.
.RE
.PD 1
.P
.PD




.TP 2
.B Function DbgLog_DnDbgLvl


.I void
.B DbgLog_DnDbgLvl
(
.I char
.B *
.B key
,
.I unsigned
.I long
.B lev
) ;

.PD 1
.P
.PD
Sets debugging levels OFF.

.B
	Args
.RS
.IP _ 2
.I key
	: filter key identifier
.IP "" 0
.IP _ 2
.I lev
	: debugging levels mask
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If the key argument is the empty string, it designs the default (generic) filter,
whose caracteristics are used as a basis for new filters, or when filter is unspecified.
When the key is NULL, it designs ALL filters except the default.
The levels mask set bits positions (as in '<<') design debugging levels set OFF.
.RE
.PD 1
.P
.PD




.TP 2
.B Function DbgLog_AddFatal


.I void
.B DbgLog_AddFatal
(
.I char
.B *
.B key
,
.I int
.B err
) ;

.PD 1
.P
.PD
Adds a fatal errno.

.B
	Args
.RS
.IP _ 2
.I key
	: filter key identifier
.IP "" 0
.IP _ 2
.I err
	: concerned errno
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If the key argument is the empty string, it designs the default (generic) filter,
whose caracteristics are used as a basis for new filters, or when filter is unspecified.
When the key is NULL, it designs ALL filters except the default.
When an error with this filter key AND this errno is encountered, this triggers
immediate termination (after the printing of a log entry), through _exit (2).
.RE
.PD 1
.P
.PD




.TP 2
.B Function DbgLog_DelFatal


.I void
.B DbgLog_DelFatal
(
.I char
.B *
.B key
,
.I int
.B err
) ;

.PD 1
.P
.PD
Deletes a fatal errno.

.B
	Args
.RS
.IP _ 2
.I key
	: filter key identifier
.IP "" 0
.IP _ 2
.I err
	: concerned errno
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
If the key argument is the empty string, it designs the default (generic) filter,
whose caracteristics are used as a basis for new filters, or when filter is unspecified.
When the key is NULL, it designs ALL filters except the default.
.RE
.PD 1
.P
.PD





.SS Log entries
.sp

.TP 2
.B Function DbgLog


.I void
.B DbgLog
(
.I const
.I char
.B *
.B mod
,
.I const
.I char
.B *
.B key
,
.I unsigned
.I char
.B lev
,
.I const
.I char
.B *
.B mes
,
.B ...
) ;

.PD 1
.P
.PD
Produces an error from module/function 'mod', filter key 'key' and message 'mes', at debugging level 'lev'.

.B
	Args
.RS
.IP _ 2
.I mod
	: name of the function/module from which this call was issued
.IP "" 0
.IP _ 2
.I key
	: key of filter to be used
.IP "" 0
.IP _ 2
.I lev
	: debugging level at which this error is raised
.IP "" 0
.IP _ 2
.I mes
	: optional message to add to transcript
.IP "" 0
.IP _ 2
.I ...
	: arguments to be formatted in the mes string
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
Use a NULL value for mes when no message needs to be printed.
Variable arguments are formatted following escapes in the message string,
taking the form '%d*?' where d* is any number of digits and ? is either:
.IP _ 2
.I %
	: a litteral '%' character is printed
.IP "" 0
.IP _ 2
.I c
	: the argument is a character
.IP "" 0
.IP _ 2
.I s
	: the argument is a string, and will be printed as-is
.IP "" 0
.IP _ 2
.I d
	: the argument is a signed number, printed in decimal form
.IP "" 0
.IP _ 2
.I u
	: the argument is an unsigned number, printed in decimal form
.IP "" 0
.IP _ 2
.I x
	: the argument is an unsigned number, printed in hexadecimal form
.IP "" 0
For s,u,x, the digits before the conversion character give the maximum
number of characters to be printed.
.RE
.PD 1
.P
.PD




.TP 2
.B Function DbgLog_


.I void
.B DbgLog_
(
.I const
.I char
.B *
.B mod
,
.I const
.I char
.B *
.B key
,
.I unsigned
.I char
.B lev
,
.I const
.I char
.B *
.B mes
,
.I va_list
.B ap
) ;

.PD 1
.P
.PD
Produces an error from module/function 'mod', filter key 'key' and message 'mes', at debugging level 'lev'.

.B
	Args
.RS
.IP _ 2
.I mod
	: name of the function/module from which this call was issued
.IP "" 0
.IP _ 2
.I key
	: key of filter to be used
.IP "" 0
.IP _ 2
.I lev
	: debugging level at which this error is raised
.IP "" 0
.IP _ 2
.I mes
	: optional message to add to transcript
.IP "" 0
.IP _ 2
.I ap
	: optional variable arguments list
.IP "" 0
.RE
.PD 1
.P
.PD


.B
	Note
.RS
Use a NULL value for mes when no message needs to be printed.
Variable arguments are formatted following escapes in the message string,
taking the form '%d*?' where d* is any number of digits and ? is either:
.IP _ 2
.I %
	: a litteral '%' character is printed
.IP "" 0
.IP _ 2
.I c
	: the argument is a character
.IP "" 0
.IP _ 2
.I s
	: the argument is a string, and will be printed as-is
.IP "" 0
.IP _ 2
.I d
	: the argument is a signed number, printed in decimal form
.IP "" 0
.IP _ 2
.I u
	: the argument is an unsigned number, printed in decimal form
.IP "" 0
.IP _ 2
.I x
	: the argument is an unsigned number, printed in hexadecimal form
.IP "" 0
For s,u,x, the digits before the conversion character give the maximum
number of characters to be printed.
This function does not call va_start (of course) and va_end, so the value of
ap is undefined after the call, and va_end must be called by the application.
.RE
.PD 1
.P
.PD





.SS Block-wise logging
.sp

.TP 2
.B Function DbgLog_OpenBlock


.I void
.B DbgLog_OpenBlock
(
.B void
) ;

.PD 1
.P
.PD
Starts a new debugging block.



.TP 2
.B Function DbgLog_CloseBlock


.I void
.B DbgLog_CloseBlock
(
.B void
) ;

.PD 1
.P
.PD
Closes the current debugging block, reactivating the last one.



.TP 2
.B Function DbgLog_Flush


.I void
.B DbgLog_Flush
(
.B void
) ;

.PD 1
.P
.PD
Prints the last debugging block in log.




.sp

.SH DOWNLOAD
The latest version of DbgLog can be obtained at 
.P
.I	http://www.varkhan.net/software/DbgLog  


.SH AUTHOR
.B	Author	:
.I	Yann LANDRIN-SCHWEITZER
, a.k.a.
.I	Varkhan
.P
.B	Contact	:
.I	varkhan@varkhan.net
.P
.B	Homepage	:
.I	http://www.varkhan.net/


.SH LICENSE

This library is free software;  you can redistribute it and/or modify it 
under  the terms of the  GNU Lesser General Public License  as published 
by the  Free Software Foundation;  either version 2.1 of the License, or 
(at your option) any later version. 

This  library  is  distributed  in  the hope that it will be useful, but 
WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser 
General Public License for more details. 

You should have received a copy of the GNU Lesser General Public License 
along with this library;  if not, write to the Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 

.SH SEE ALSO
.B DbgCfg(3), DbgCfg(5)


