.TH DbgLog 3 "April 2007" Unix Libraries
.SH NAME
.I DbgLog
-  Advanced Debugging Messages Filter and Logger

.SH DESCRIPTION
  The
.B DbgLog
library filters and logs debugging/error messages according to
.I errno
, calling module/function, warning keyword and debugging level.
Particular errno/keyword can be made fatal. Output log files depend on
keyword.

  Typical output looks like:

.PD 0
.RE
IPC.....<04>\ 1016120237s|706271us\ [000004]\ :\ 0003\ @\ main\ >\ \ ///
.P
-----------------------------------------------------------
.P
  dbg    dbg         time           msg     errno  module
.P
 keywd  level    seconds|milisec   number

.sp
///\ No\ such\ process\ (0000000543)
.P
    ----------------------------
.P
     errno message    opt args
.RS
.PD

  But the output format can also be changed at will by setting a format
string.


  This tool offers a tunable insight into the hierarchy of methods called
within the libraries using it. It provides a way to trace and debug invalid 
data as well as misbehaving functions. The ability to turn off or filter 
whole segments of the debugging traces ensures that debugging will not be 
hindered by useless data. It filters output much in the same way as system 
calls debugging tools (such as strace) allow. 


  The API definition is entirely contained in the
.I <DbgLog.h>
include file.

  From the programmer's angle, the main entry point in the API is the DbgLog
funtion, that generates a log entry. To do that, this call must be supplied
with contextual information, like function name, type of entry, and severity
level. While this may seem cumbersome at first, this call needs only be used
in error contexts, meaning its use is not too frequent. Here is the synopsis
(this is a variable argument count function):


.I void
.B DbgLog
.R (
.I const char *
.R mod,
.I const char *
.R key,
.I unsigned char
.R lev,
.I const char *
.R mes, ...) ;

.I mod
is the calling module or function name. This is the main context
data, as this information will structure the actual hierarchy of calls.
.I key
is a debugging key, an identifier tag that will direct logging and
filtering of this message.
.I lev
is a numeric indication of severity (the lower, the more severe, with
0 having sense as "system-level error").

  From the user's angle (that is, the user of a library compiled with DbgLog),
several calls provide the tools to fine-tune the logging output:
.BR DbgLog_SetFormat ", " DbgLog_SetLogFd ", " DbgLog_UpDbgLvl " and " DbgLog_DnDbgLvl ", " DbgLog_AddFatal " and " DbgLog_DelFatal.
	
.B   DbgLog_SetFormat
has a single string argument, determining the format
of log entries. The remaining functions have a debug key as first argument,
standing for a particular class of filtering and, more generally, handling
of that class of messages. The second argument is, for
.B DbgLog_SetLogFd
, an output file descriptor in which the messages will be written. The
.B ...DbgLvl
calls use a binary mask of levels (between 0 and 31) to set on or off. The
.B ...Fatal
calls set or unset some system error values to trigger immediate termination.


.SH DOWNLOAD
The latest version of DbgLog can be obtained at 
.P
.I	http://www.varkhan.net/software/DbgLog  


.SH AUTHOR
.B	Author	:
.I	Yann LANDRIN-SCHWEITZER
, a.k.a.
.I	Varkhan
.P
.B	Contact	:
.I	varkhan@varkhan.net
.P
.B	Homepage	:
.I	http://www.varkhan.net/


.SH TODO
Beats me.


.SH LICENSE

This library is free software;  you can redistribute it and/or modify it 
under  the terms of the  GNU Lesser General Public License  as published 
by the  Free Software Foundation;  either version 2.1 of the License, or 
(at your option) any later version. 

This  library  is  distributed  in  the hope that it will be useful, but 
WITHOUT   ANY   WARRANTY;   without   even   the   implied  warranty  of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser 
General Public License for more details. 

You should have received a copy of the GNU Lesser General Public License 
along with this library;  if not, write to the Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 

.SH SEE ALSO
.B DbgLog_api(3)

